<?php

namespace Drupal\browser_development\Processing;

/**
 * Class FileSystemStructure controls the file system array.
 *
 * @todo Have the ability to change the paths need alter where items are
 * compiled and saved on the file system.
 *
 * @package Drupal\browser_development\Processing
 */
class FileSystemStructure {

  /**
   * Global array that holds all relevant paths.
   *
   * @var array
   *   Sets global path.
   */
  public $globalFilePathArray = [
    'base_path' => 'public://browser-development/',
    'css_directory' => 'css',
    'css_name' => 'browser-development.css',
    'scss_directory' => 'scss',
  ];

  /**
   * File system full path.
   *
   * @var string
   *   Finds absolute path for CSS directory.
   */
  public $absolutePath;

  /**
   * Customers CSS styles.
   */
  public function __construct() {
    if (!empty(FormsStorage::getStorage('browser_development_settings')['browser_development_settings'])) {
      $this->globalFilePathArray['base_path'] = FormsStorage::getStorage('browser_development_settings')['browser_development_settings']['base_path'];
    }
    $this->getAbsolutePath();
    $this->createFileAndDirectoryStructure();
  }

  /**
   * Gets Absolute Path for CSS file creation.
   */
  private function getAbsolutePath() {

    $this->absolutePath = \Drupal::service('file_system')->realpath(
            $this->globalFilePathArray['base_path']);
  }

  /**
   * Creates public directory for CSS file if needed .
   */
  private function createFileAndDirectoryStructure() {

    $public = $this->globalFilePathArray['base_path'];
    $css = $this->globalFilePathArray['css_directory'];
    $file = $this->globalFilePathArray['css_name'];
    $scss = $this->globalFilePathArray['scss_directory'];

    if (!is_dir($public)) {
      $fileSystem = \Drupal::service('file_system');
      $fileSystem->mkdir($public, 0775);
      $fileSystem->mkdir($public . $css, 0775);
      $fileSystem->mkdir($public . $scss, 0775);
      $path = $public . DIRECTORY_SEPARATOR . $css . DIRECTORY_SEPARATOR . $file;
      $fileOpen = fopen($path, "wb");
      fwrite($fileOpen, "{}");
      fclose($fileOpen);
    }
  }

  /**
   *
   * @param array $path
   * @return array
   */
  public function setPathArray($path = []) {

    if (!empty($path)) {
      $this->globalFilePathArray['base_path'] = $path;

      $arr['base_path'] = $this->globalFilePathArray['base_path'];
      FormsStorage::setStorage('browser_development_settings', $arr);
    }

  }

  /**
   * @return array
   */
  public function getPathArray() {

    return $this->globalFilePathArray;
  }

}
